function [z] = covarK(b, opts)
%function [z] = covarK(b, opts)
%
% COVARK computes the matrix-vector product z = K*b, where K is the
% covariance matrix specified by the option opts, and b is a
% right-hand vector.
%

if ~exist('opts', 'var')
  opts = struct;
end
m1 = setparam(opts, 'm1', 100);
m2 = setparam(opts, 'm2', 100);
alpha = setparam(opts, 'alpha', 12.5);
j = setparam(opts, 'j', 3);
deform = setparam(opts, 'deform', 'none');

% window
falpha = floor(alpha);
window = repmat([-falpha:falpha]', 1, 2*falpha+1) + ...
         repmat([-falpha:falpha]*m1, 2*falpha+1, 1);

% distances inside window
x = repmat([-falpha:falpha], 2*falpha+1, 1);
y = repmat([-falpha:falpha]', 1, 2*falpha+1);
if strcmp(deform, 'scaley')
  suppx=max(max(x));
  inffx=min(min(x));
  mullx=2*(x-ones(size(x))*(suppx+inffx)/2)/(suppx-inffx);
  mullx=2*ones(size(x))-mullx.*mullx;
else
  mullx = ones(size(y));
end
wdist = sqrt(x.^2+(mullx.*y).^2);

% trim out the points farther than distance alpha
window = window(:)';
wdist = wdist(:)';
x = x(:)';
y = y(:)';
ind = find(wdist<=alpha);
window = window(ind);
wdist = wdist(ind);
x = x(ind);
y = y(ind);

% entry values of K (a row)
Kentry = (1-wdist./alpha).^(j+1);

% compute the mat-vec
%--the rest of the code is replaced by mex function covar_mat_vec
z = covarK_mat_vec(m1, m2, falpha, window, x, y, Kentry, b);


%{
m = m1*m2;
z = zeros(m,1);
i = 0;
full_ind = 1:length(window);
ii_cut1 = falpha;
ii_cut2 = m1-falpha;
jj_cut1 = falpha;
jj_cut2 = m2-falpha;
for jj = 1:m2
  for ii = 1:m1
    
    i = i + 1;
  
    % neighbors
    neighbors = i + window;
  
    % find out the real neighbors; trim out the ghosts
    if ii>ii_cut1 && ii<=ii_cut2 && jj>jj_cut1 && jj<=jj_cut2 %center
      ind = full_ind;
    elseif ii<=ii_cut1 && jj> jj_cut1 && jj<=jj_cut2 %north
      ind = find(y > -ii);
    elseif jj<=jj_cut1 && ii> ii_cut1 && ii<=ii_cut2 %west
      ind = find(x > -jj);
    elseif ii> ii_cut2 && jj> jj_cut1 && jj<=jj_cut2 %south
      ind = find(y <= m1-ii);
    elseif jj> jj_cut2 && ii> ii_cut1 && ii<=ii_cut2 %east
      ind = find(x <= m2-jj);
    elseif ii<=ii_cut1 && jj<=jj_cut1 %northwest
      ind = find(y > -ii & x > -jj);
    elseif ii> ii_cut2 && jj<=jj_cut1 %southwest
      ind = find(y <= m1-ii & x > -jj);
    elseif ii> ii_cut2 && jj> jj_cut2 %southeast
      ind = find(y <= m1-ii & x <= m2-jj);
    elseif ii<=ii_cut1 && jj> jj_cut2 %northeast
      ind = find(y > -ii & x <= m2-jj);
    end
  
    % a row times a column
    z(i) = Kentry(ind) * b(neighbors(ind));
  
  end
end
%}




%--------------------------------------------------------------
function x = setparam(opts, name, default)

if isfield(opts, name)
  x = opts.(name);
else
  x = default;
end
