#include "math.h"
#include "mex.h"

/* called by covarK.m */
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
  /* input variables */
  int m1, m2, falpha;
  double *windowd, *xd, *yd;
  double *Kentry, *b;

  /* output variables */
  double *z;

  /* other variables */
  int m, len;
  int i, ii, jj, kk;

  /* grab inputs */
  m1      = (int)mxGetScalar(prhs[0]);
  m2      = (int)mxGetScalar(prhs[1]);
  falpha  = (int)mxGetScalar(prhs[2]);
  windowd = mxGetPr(prhs[3]);   /* lengths of window, x, y, Kentry */
  len     = mxGetN(prhs[3]);    /* should all be the same */
  xd      = mxGetPr(prhs[4]);
  yd      = mxGetPr(prhs[5]);
  Kentry  = mxGetPr(prhs[6]);
  b       = mxGetPr(prhs[7]);
  m       = mxGetM(prhs[7]);    /* m should be equal to m1*m2 */

  int *window = (int*)calloc(len, sizeof(int));
  for (i = 0; i < len; i++)
    window[i] = (int)(windowd[i]);
  int *x = (int*)calloc(len, sizeof(int));
  for (i = 0; i < len; i++)
    x[i] = (int)(xd[i]);
  int *y = (int*)calloc(len, sizeof(int));
  for (i = 0; i < len; i++)
    y[i] = (int)(yd[i]);

  /* allocate outputs */
  plhs[0] = mxCreateDoubleMatrix(m, 1, mxREAL);
  z       = mxGetPr(plhs[0]);

  /* main computations */
  int ii_cut1 = falpha;
  int ii_cut2 = m1-falpha;
  int jj_cut1 = falpha;
  int jj_cut2 = m2-falpha;

  int *neighbors = (int*)calloc(len, sizeof(int));

  i = 0;
  for (jj = 1; jj <= m2; jj++)
    for (ii = 1; ii <= m1; ii++)
      {
        for (kk = 0; kk < len; kk++)
          neighbors[kk] = i + window[kk];

        if (ii>ii_cut1 && ii<=ii_cut2 && jj>jj_cut1 && jj<=jj_cut2) /* center */
          {
            for (kk = 0; kk < len; kk++)
              z[i] += Kentry[kk] * b[ neighbors[kk] ];
          }
        else if (ii<=ii_cut1 && jj> jj_cut1 && jj<=jj_cut2) /* north */
          {
            for (kk = 0; kk < len; kk++)
              if (y[kk] > -ii)
                z[i] += Kentry[kk] * b[ neighbors[kk] ];
          }
        else if (jj<=jj_cut1 && ii> ii_cut1 && ii<=ii_cut2) /* west */
          {
            for (kk = 0; kk < len; kk++)
              if (x[kk] > -jj)
                z[i] += Kentry[kk] * b[ neighbors[kk] ];
          }
        else if (ii> ii_cut2 && jj> jj_cut1 && jj<=jj_cut2) /* south */
          {
            for (kk = 0; kk < len; kk++)
              if (y[kk] <= m1-ii)
                z[i] += Kentry[kk] * b[ neighbors[kk] ];
          }
        else if (jj> jj_cut2 && ii> ii_cut1 && ii<=ii_cut2) /* east */
          {
            for (kk = 0; kk < len; kk++)
              if (x[kk] <= m2-jj)
                z[i] += Kentry[kk] * b[ neighbors[kk] ];
          }
        else if (ii<=ii_cut1 && jj<=jj_cut1) /* northwest */
          {
            for (kk = 0; kk < len; kk++)
              if (y[kk] > -ii && x[kk] > -jj)
                z[i] += Kentry[kk] * b[ neighbors[kk] ];
          }
        else if (ii> ii_cut2 && jj<=jj_cut1) /* southwest */
          {
            for (kk = 0; kk < len; kk++)
              if (y[kk] <= m1-ii && x[kk] > -jj)
                z[i] += Kentry[kk] * b[ neighbors[kk] ];
          }
        else if (ii> ii_cut2 && jj> jj_cut2) /* southeast */
          {
            for (kk = 0; kk < len; kk++)
              if (y[kk] <= m1-ii && x[kk] <= m2-jj)
                z[i] += Kentry[kk] * b[ neighbors[kk] ];
          }
        else if (ii<=ii_cut1 && jj> jj_cut2) /* northeast */
          {
            for (kk = 0; kk < len; kk++)
              if (y[kk] > -ii && x[kk] <= m2-jj)
                z[i] += Kentry[kk] * b[ neighbors[kk] ];
          }
        
        i++;
      }

  /* free memory */
  free(neighbors);
  free(window);
  free(x);
  free(y);

  return;
}
