% This example shows how to compute z = sqrt(A)*b where A is a
% covariance matrix. Here A is in an operator form (a function), which
% is defined in covarK.m, which calls a subroutine
% covarK_mat_vec.mexa64. This subroutine was compiled from
% covarK_mat_vec.c using the `mex' utility in matlab. Since A is
% not a matrix, we cannot compute the residual.

clear

% prepare parameters. No need to explicitly generate A.
f = @sqrt;
k = 100;
tol = 1e-16;

% get intervals
opts = struct('m1',1e2, 'm2',1e2, 'alpha', 6.5, 'j',3, 'deform','none');
m = opts.m1 * opts.m2;
[min_eig, max_eig] = eigk(@(x)covarK(x, opts), m);
t = get_intervals_geometric(min_eig, max_eig);
b = ones(m,1);

% compute approximation
[z,z_dif,res,num_iter] = ...
    stieltjes_approx(@(x)covarK(x, opts), b, f, t, k, [], tol);

% plot
semilogy(z_dif(1:num_iter),'r')
legend('z dif');



