% This example shows how to compute z = exp(A)*b where A is a 2D
% Laplacian. A different thing from sqrt is that we need to scale A
% such that its spectral radius is 1. Another thing is that we use a
% different interval scheme.

clear

% generate data
f = @exp;
[A, b, fAb, min_eig, max_eig] = gen_data('lap2D', f);
%--scale the matrix. Recall that A is diagonal--
[ii,jj,vv] = find(A);
A = A/max(vv);
fAb = f(vv/max(vv)).*b;
min_eig = min_eig/max(vv);
max_eig = 1;
%-----------------------------------------------
m = length(b);
t = get_intervals_linear(min_eig, max_eig, m);
k = 100;
tol = 1e-16;

% compute approximation
[z,z_dif,res,num_iter] = stieltjes_approx(A, b, f, t, k, fAb, tol);

% plot
semilogy(res(1:num_iter))
hold on
semilogy(z_dif(1:num_iter),'r')
hold off
legend('res', 'z dif');





